@@ -122,11 +122,9 @@ ps_begin_path(double width)	// Q: what's with this width=-1 condition??
 		fprintf(_grPS, "] %d d\n", int(_dash.size()));
 		break;
 	case svg:
-		fprintf(_grSVG, "<g>\n<path style=\"stroke:#%02x%02x%02x; stroke-width:%.3f; fill:none;stroke-opacity:%f;stroke-linejoin:%s;stroke-linecap:%s\"\nd=\"\n",
-			int(255.0*_griState.color_line().getR()),
-			int(255.0*_griState.color_line().getG()),
-			int(255.0*_griState.color_line().getB()),
-			_griState.linewidth_line(),
+		fprintf(_grSVG, "<g>\n<path style=\"stroke:%s; stroke-width:%.3f; fill:none;stroke-opacity:%f;stroke-linejoin:%s;stroke-linecap:%s\"\nd=\"\n",
+			_griState.color_line().get_hexcolor().c_str(),
+			(width == -1) ? _griState.linewidth_line() : width,
 			1.0 - _griState.color_line().getT(),
 			_griState.line_join() == 0 ? "sharp" : (_griState.line_join() == 1 ? "round" : "bevel"),
 			_griState.line_cap()  == 0 ? "butt"  : (_griState.line_cap()  == 1 ? "round" : "square"));
@@ -294,7 +292,12 @@ void GriPath::stroke_or_fill(char s_or_f, units the_units, double width, bool cl
 				fprintf(_grPS, "%% END GriPath stroke/fill\n");
 				break;
 			case svg:
-				fprintf(stderr, "%s:%d:GriPath.cc SVG is not being told the _type_ of path (filled/stroked, etc)\n", __FILE__,__LINE__);
+				if (s_or_f == 'f') {
+					fprintf(stderr, "%s:%d ERROR - cannot fill paths in SVG mode\n", __FILE__,__LINE__);
+				} else {
+					if (closepath)
+						fprintf(stderr, "%s:%d ERROR - cannot close paths in SVG mode\n", __FILE__,__LINE__);
+				}
 				fprintf(_grSVG, "\"/>\n</g>\n");
 				break;
 			case gif:
