@@ -50,29 +50,33 @@ bool            set_x_scale(void);
 void
 set_line_width_symbol()
 {
-	double          linewidthsymbol = LINEWIDTHSYMBOL_DEFAULT;
+	double linewidthsymbol = LINEWIDTHSYMBOL_DEFAULT;
 	if (!get_var("..linewidthsymbol..", &linewidthsymbol))
 		warning("Sorry ..linewidthsymbol.. undefined so using default");
 	_griState.set_linewidth_symbol(linewidthsymbol);
-	extern FILE *_grPS;
-	fprintf(_grPS, "%.3f w\n", _griState.linewidth_symbol());
+	if (_output_file_type == postscript) {
+		extern FILE *_grPS;
+		fprintf(_grPS, "%.3f w\n", _griState.linewidth_symbol());
+	}
 }
 
 void
 set_line_width_curve()
 {
-	double          linewidth = LINEWIDTH_DEFAULT;
+	double linewidth = LINEWIDTH_DEFAULT;
 	if (!get_var("..linewidth..", &linewidth))
 		warning("Sorry ..linewidth.. undefined so using default");
 	_griState.set_linewidth_line(linewidth);
-	extern FILE *_grPS;
-	fprintf(_grPS, "%.3f w\n", _griState.linewidth_line());
+	if (_output_file_type == postscript) {
+		extern FILE *_grPS;
+		fprintf(_grPS, "%.3f w\n", _griState.linewidth_line());
+	}
 }
 
 void
 set_line_width_axis()
 {
-	double          linewidthaxis = LINEWIDTHAXIS_DEFAULT;
+	double linewidthaxis = LINEWIDTHAXIS_DEFAULT;
 	if (!get_var("..linewidthaxis..", &linewidthaxis))
 		warning("Sorry ..linewidthaxis.. undefined so using default");
 	_griState.set_linewidth_axis(linewidthaxis);
@@ -1299,7 +1303,11 @@ draw_curveCmd()
 	double          lastx = gr_currentmissingvalue();
 	double          lasty = gr_currentmissingvalue();
 	bool            first = true, last_OK = true;
+	extern FILE *_grSVG;
 	unsigned int i_max = _colX.size();
+	if (_output_file_type == svg) {
+		fprintf(_grSVG, "<g> <!-- %s -->\n", _cmdLine);
+	}
 	set_environment();
 	set_line_width_curve();
 	GriPath  path(i_max);
@@ -1336,6 +1344,9 @@ draw_curveCmd()
 	PUT_VAR("..ylast..", lasty);
 	_drawingstarted = true;
 	draw_axes_if_needed();
+	if (_output_file_type == svg) {
+		fprintf(_grSVG, "</g> <!-- %s -->\n", _cmdLine);
+	}
 	return true;
 }
 
@@ -1855,6 +1866,9 @@ Note the extra word `box', and the new meaning of the last 2 parameters.");
 bool
 draw_image_paletteCmd()
 {
+	if (_output_file_type == svg)
+		warning("svg mode may not handle image palettes properly yet");
+	extern FILE    *_grPS;
 	const double height = 1.0;	// height of box (cm)
 	const double space = 2.0;	// space of box above top of plot
 	const int LEN = 512;	// length of tmp image
@@ -1994,14 +2008,15 @@ draw_image_paletteCmd()
 		gr_usertocm(1.0, right, &right_cm, &top_cm);
 	}
 	// Clip to this, because image overhangs
-	extern FILE    *_grPS;
-	fprintf(_grPS, "q n %% turn clipping on for image palette\n");
-	fprintf(_grPS, "%f %f moveto\n", left_cm * PT_PER_CM, bottom_cm * PT_PER_CM);
-	fprintf(_grPS, "%f %f lineto\n", right_cm * PT_PER_CM, bottom_cm * PT_PER_CM);
-	fprintf(_grPS, "%f %f lineto\n", right_cm * PT_PER_CM, top_cm * PT_PER_CM);
-	fprintf(_grPS, "%f %f lineto\n", left_cm * PT_PER_CM, top_cm * PT_PER_CM);
-	fprintf(_grPS, "%f %f lineto\n", left_cm * PT_PER_CM, bottom_cm * PT_PER_CM);
-	fprintf(_grPS, "closepath W\n");
+	if (_output_file_type == postscript) {
+		fprintf(_grPS, "q n %% turn clipping on for image palette\n");
+		fprintf(_grPS, "%f %f moveto\n", left_cm * PT_PER_CM, bottom_cm * PT_PER_CM);
+		fprintf(_grPS, "%f %f lineto\n", right_cm * PT_PER_CM, bottom_cm * PT_PER_CM);
+		fprintf(_grPS, "%f %f lineto\n", right_cm * PT_PER_CM, top_cm * PT_PER_CM);
+		fprintf(_grPS, "%f %f lineto\n", left_cm * PT_PER_CM, top_cm * PT_PER_CM);
+		fprintf(_grPS, "%f %f lineto\n", left_cm * PT_PER_CM, bottom_cm * PT_PER_CM);
+		fprintf(_grPS, "closepath W\n");
+	}
 	if (rotpal==0) {
 		gr_drawimage(gray,
 			     _imageTransform, 
@@ -2019,7 +2034,8 @@ draw_image_paletteCmd()
 			     left_cm, bottom_cm, right_cm, top_cm,
 			     false);
 	}
-	fprintf(_grPS, "Q %% turn clipping off for image palette\n");
+	if (_output_file_type == postscript)
+		fprintf(_grPS, "Q %% turn clipping off for image palette\n");
 	double actual_linewidth = _griState.linewidth_line();
 	_griState.set_linewidth_line(_griState.linewidth_axis());
 	if (rotpal==0) {
@@ -2027,7 +2043,8 @@ draw_image_paletteCmd()
 			gr_drawxaxis(0.0, left, inc, right, gr_axis_BOTTOM);
 		else
 			gr_drawxaxis(1.0, left, inc, right, gr_axis_TOP);
-		fprintf(_grPS, "%.3f w %% test\n", _griState.linewidth_axis());
+		if (_output_file_type == postscript)
+			fprintf(_grPS, "%.3f w %% test\n", _griState.linewidth_axis());
 		GriPath p(4);
 		p.push_back(left, 0.0, 'm');
 		p.push_back(left, 1.0, 'l');
@@ -2040,7 +2057,8 @@ draw_image_paletteCmd()
 			gr_drawyaxis(0.0, left, inc, right, gr_axis_LEFT);
 		else
 			gr_drawyaxis(1.0, left, inc, right, gr_axis_RIGHT);
-		fprintf(_grPS, "%.3f w\n", _griState.linewidth_axis());
+		if (_output_file_type == postscript)
+			fprintf(_grPS, "%.3f w\n", _griState.linewidth_axis());
 		GriPath p(4);
 		p.push_back(0.0, left, 'm');
 		p.push_back(1.0, left, 'l');
@@ -2064,13 +2082,8 @@ draw_imageCmd()
 		return false;
 	}
 	if (!scales_defined()) {
-#if 0				// Fix SF bug #129856 (I hope!)
-		no_scales_error();
-		return false;
-#else
 		create_x_scale();
 		create_y_scale();
-#endif
 	}
 	if (!_image.storage_exists) {
 		demonstrate_command_usage();
@@ -2096,17 +2109,39 @@ draw_imageCmd()
 		mask_g = _image_missing_color_green;
 		mask_b = _image_missing_color_blue;
 		//printf("%s:%d image mask at %x\n",__FILE__,__LINE__,(unsigned int)( _imageMask.image));
-		gr_drawimage(_image.image, _imageTransform, _image_color_model,
-			     _imageMask.image, mask_r, mask_g, mask_b,
-			     _image.ras_width, _image.ras_height,
-			     llx_cm, lly_cm, urx_cm, ury_cm,
-			     true);
+		if (_output_file_type == svg)
+			gr_drawimage_svg(_image.image,
+					 _imageTransform, 
+					 _image_color_model,
+					 _imageMask.image, mask_r, mask_g, mask_b,
+					 _image.ras_width, _image.ras_height,
+					 llx_cm, lly_cm, urx_cm, ury_cm,
+					 true);
+		else 
+			gr_drawimage(_image.image, 
+				     _imageTransform, 
+				     _image_color_model,
+				     _imageMask.image, mask_r, mask_g, mask_b,
+				     _image.ras_width, _image.ras_height,
+				     llx_cm, lly_cm, urx_cm, ury_cm,
+				     true);
 	} else {
-		gr_drawimage(_image.image, _imageTransform, _image_color_model,
-			     NULL, 0.0, 0.0, 0.0,
-			     _image.ras_width, _image.ras_height,
-			     llx_cm, lly_cm, urx_cm, ury_cm,
-			     true);
+		if (_output_file_type == svg)
+			gr_drawimage_svg(_image.image,
+					 _imageTransform, 
+					 _image_color_model,
+					 NULL, 0.0, 0.0, 0.0,
+					 _image.ras_width, _image.ras_height,
+					 llx_cm, lly_cm, urx_cm, ury_cm,
+					 true);
+		else 
+			gr_drawimage(_image.image,
+				     _imageTransform, 
+				     _image_color_model,
+				     NULL, 0.0, 0.0, 0.0,
+				     _image.ras_width, _image.ras_height,
+				     llx_cm, lly_cm, urx_cm, ury_cm,
+				     true);
 	}
 	_drawingstarted = true;
 	draw_axes_if_needed();
