@@ -1,6 +1,6 @@
 #include	"extern.hh"
 extern FILE *_grSVG;
-
+static std::vector<std::string> group_name;
 
 bool
 groupCmd()
@@ -18,10 +18,12 @@ bool
 group_start(const char *id)
 {
 	if (_output_file_type == svg) {
+		std::string name = id;
 		if (strlen(id) > 0)
-			fprintf(_grSVG, "<g id=\"%s\">\n", id);
+			fprintf(_grSVG, "<g> <!-- %s -->\n", id);
 		else
-			fprintf(_grSVG, "<g>\n");
+			fprintf(_grSVG, "<g> <!-- anonymous group -->\n");
+		group_name.push_back(name);
 	}
 	return true;
 }
@@ -29,7 +31,12 @@ bool
 group_end()
 {
 	if (_output_file_type == svg) {
-		fprintf(_grSVG, "</g>\n");
+		if (group_name.size() > 0) {
+			fprintf(_grSVG, "</g> <!-- end of %s -->\n", group_name.back().c_str());
+			group_name.pop_back();
+		} else {
+			fprintf(_grSVG, "</g> <!-- end of anonymous group -->\n");
+		}
 	}
 	return true;
 }
