@@ -7,8 +7,10 @@
 #include <math.h>
 #include <stdio.h>
 #include <stdarg.h>
+#include <limits.h>
 #if !defined(IS_MINGW32)
 #include <pwd.h>
+#include <paths.h>
 #endif
 #include <sys/types.h>
 
@@ -428,10 +430,13 @@ word_is(int i, const char *word)
 void
 check_psfile()
 {
-	extern FILE    *_grPS;
-	if (ferror(_grPS)) {
-		/* never returns */
-		fatal_err(" IO error on PostScript output file\n");
+	extern output_file_type _output_file_type;
+	if (_output_file_type == postscript) {
+		extern FILE *_grPS;
+		if (ferror(_grPS)) {
+			/* never returns */
+			fatal_err(" IO error on PostScript output file\n");
+		}
 	}
 }
 
@@ -1983,21 +1988,24 @@ vector_max(double *v, unsigned n)
 void
 set_ps_color(char what)		// what='p' for path or 't' for text
 {
-	extern FILE *_grPS;
-	extern bool _grWritePS;
-	if (!_grWritePS)
-		return;
-	double r, g, b;
-	if (what == 'p')
-		_griState.color_line().getRGB(&r, &g, &b);
-	else
-		_griState.color_text().getRGB(&r, &g, &b);
-	if (r == g && g == b) {
-		fprintf(_grPS, "%.3g g\n", r);
-		fprintf(_grPS, "%.3g G\n", r);
-	} else {
-		fprintf(_grPS, "%.3g %.3g %.3g rg\n", r, g, b);
-		fprintf(_grPS, "%.3g %.3g %.3g RG\n", r, g, b);
+	extern output_file_type _output_file_type;
+	if (_output_file_type == postscript) {
+		extern FILE *_grPS;
+		extern bool _grWritePS;
+		if (!_grWritePS)
+			return;
+		double r, g, b;
+		if (what == 'p')
+			_griState.color_line().getRGB(&r, &g, &b);
+		else
+			_griState.color_text().getRGB(&r, &g, &b);
+		if (r == g && g == b) {
+			fprintf(_grPS, "%.3g g\n", r);
+			fprintf(_grPS, "%.3g G\n", r);
+		} else {
+			fprintf(_grPS, "%.3g %.3g %.3g rg\n", r, g, b);
+			fprintf(_grPS, "%.3g %.3g %.3g RG\n", r, g, b);
+		}
 	}
 }
 
@@ -2182,6 +2190,23 @@ lapse_rate(double S, double t, double p)
 char*
 tmp_file_name()
 {
+#if defined(HAVE_MKSTEMP)
+	static char rval[PATH_MAX];
+	int fd;
+	/*
+	 * Create the file safely and let caller scribble on it. Not
+	 * perfect but better than the alternative. (One could also
+	 * change the function to return a fd or FILE * instead of a
+	 * path, but that's quite invasive.)
+	 */
+	strcpy(rval, _PATH_TMP "griXXXXXX");
+	fd = mkstemp(rval);
+	if (fd < 0) {
+		return NULL;
+	}
+	close(fd);
+	return rval;
+#else
 #if defined(HAVE_TEMPNAM)
 	//	rval = tempnam("/usr/tmp", "gri");
 	char *rval = tempnam(NULL, "gri");
@@ -2198,6 +2223,7 @@ tmp_file_name()
 	return GRI_TMP_FILE;
 #endif
 #endif
+#endif
 }
 
 int
